#include <stdio.h>
#include <stdbool.h>
#include <termios.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "list.h"

#define DEBUG 0
#define MAX_PASSWORD_LEN 64
#define MAX_COMMAND_LEN 10240 //could be less, could care less

void askpass(char *prompt);
static int callTruecrypt(char *arg,char *sPass);

char sBuffer[MAX_PASSWORD_LEN+1];

int main(int argc, char *argv[])
{
	int i;
	listNode *root,*p;

	root=(listNode*)malloc(sizeof(listNode)); 
	root->next=NULL;
	root->sData=NULL;
	if(DEBUG==1)	
	{
		for(i=1;i<argc;i++)
		{
			printf("DST: %s\n",argv[i]);
			i++;
			if(i>=argc)
				break;
			printf("SRC: %s\n",argv[i]);
			i++;
			if(i>=argc)
				break;
			printf("CALL: %s\n",argv[i]);
		}
	}
	for(i=1;i<argc;i++)
	{
		char *sMountDest=argv[i];
		i++;
		if(i>=argc)
		{
			printf("FORMAT ERROR\n");
			exit(23);
		}
		//char *sMountSrc=argv[i];
		i++;
		if(i>=argc)
		{
			printf("FORMAT ERROR\n");
			exit(23);
		}
		char *sMountCall=argv[i];

		int bSuccess=0;
		p=root;
		while(p->next!=NULL)
		{
			p=p->next;
			if(p->bIgnore==0&&callTruecrypt(sMountCall,p->sData)==0)
			{
				printf("Processing '%s': DONE.\n",sMountDest);
				bSuccess=1;
				break;
			}
		}
		if(bSuccess==1)
			continue;
		bSuccess=1;
		
		listNode* newPass=NULL;
		do
		{
			if(bSuccess==0)
			{
				if(newPass!=NULL)
					newPass->bIgnore=1;
				askpass("\tInvalid password, retry again: ");
				bSuccess=1;//premature
			}
			else
			{
				char *s=malloc(MAX_COMMAND_LEN); 
				snprintf(s,MAX_COMMAND_LEN,"Processing '%s': enter passphrase: ",sMountDest);
				askpass(s);
				free(s);
			}
			if(strcmp(sBuffer,"")==0)
			{
				printf("\tSKIPPED!\n");
				bSuccess=-1;
				break;
			}
			newPass=uniqueInsert(root,sBuffer);
			bSuccess=0;
		}
		while(callTruecrypt(sMountCall,sBuffer)!=0);
		if(bSuccess!=-1)
		{
			printf("\tDONE!\n");
			newPass->bIgnore=0;
		}
		else if(newPass!=NULL) 
			newPass->bIgnore=1;
	}
	destructList(root);
}

static int callTruecrypt(char *sArg,char *sPass)
{
	char *sTemp=malloc(MAX_COMMAND_LEN); 
	snprintf(sTemp,MAX_COMMAND_LEN,"ulimit -c 0;printf \"%%s\" \"%s\" | %s >> /dev/null 2>&1",sPass,sArg);//ulimit, disable core dump creation 
	int iResult=system(sTemp);
	memset(sTemp,0,sizeof(sTemp));//overwrite memory for safety
	__asm__ volatile("" : : : "memory");//stops gcc 4.7.2? from eliminating memset in optimiziation stage
	free(sTemp);
	return iResult;
}
/* Copyright (c) 2006-2011 Dovecot authors, see the included COPYING file 
//MIT license for askpass_str
*/
static void askpass_str(const char *prompt, char *pass)
{
	struct termios old_tio,tio;
	bool tty,restore_tio=0;
	char ch;
	int fd;
	char *p=pass;

	tty=isatty(STDIN_FILENO);
	if(tty) 
	{
		fputs(prompt,stderr);
		fflush(stderr);

		fd=open("/dev/tty",O_RDONLY);
		if(fd<0)
			perror("open(/dev/tty) failed");

		if(tcgetattr(fd,&old_tio)==0)//turn off echo
		{
			restore_tio=1;
			tio=old_tio;
			tio.c_lflag&=~(ECHO|ECHONL);
			(void)tcsetattr(fd,TCSAFLUSH,&tio);
		}
	} 
	else 
	{
		fd=STDIN_FILENO;//read it from stdin without showing a prompt
	}
	//read the password
	while(read(fd,&ch,1)>0)
	{
		if(ch=='\n'||ch=='\r')
			break;
		if(p-pass>=MAX_PASSWORD_LEN)
			break;
		else
			*p++=ch;
	}
	if(tty) 
	{
		if(restore_tio)
			(void)tcsetattr(fd,TCSAFLUSH,&old_tio);
		fputs("\n", stderr);
		fflush(stderr);
		(void)close(fd);
	}
	*p++='\0';
}

void askpass(char *prompt)
{
	askpass_str(prompt,sBuffer);
}
